# Author: Gaoxiang Wang
import pandas as pd 
import altair as alt
from panel.interact import interact
import panel as pn
import numpy as np


def soc_viz_stats_scatter(x_col, y_col,df):
    """Create a scatter plot of two chosen numerical features.
    
    Parameters
    ----------
    x_col : str
        name of the feature in horizontal axis. 
    y_col : str
        name of the feature in vertical axis.
    df : pandas dataframe
        The dataframe of the feature

    Returns
    -------
    Chart: altair.vegalite.v4.api.Chart
        an altair chart
    Examples
    --------
    >>> data = pd.read_excel("soccer_data.xlsx")
    >>> soc_viz_stats_scatter('age', 'Wages_Euros', data)
    """
    assert isinstance(x_col,str) == True, "Column named used should be passed as string"
    assert (x_col in df.columns) == True, f'Column {x_col} does not exist in data frame'
    assert np.issubdtype(df[x_col].dtype, np.number) ==True, "Column used should be numeric"
    assert isinstance(y_col,str) == True, "Column named used should be passed as string"
    assert (y_col in df.columns) == True, f'Column {y_col} does not exist in data frame'
    assert np.issubdtype(df[y_col].dtype, np.number) ==True, "Column used should be numeric"
    assert set(['Name','age', 'Team','Continent','Position_Final']).issubset(set(df.columns)),  "'Name','age', 'Team','Continent','Position_Final' shoud be included "

    
    slider = alt.binding_range(
        name='age', step=1,
        min=df['age'].min(), max=df['age'].max())
    select_age = alt.selection_single(
        fields=['age'],
        bind=slider,
        init={'age': 18})
    chart = alt.Chart(df).mark_point().encode(
        x=x_col,
        y=y_col,
        tooltip=['Name','age', 'Team','Continent','Position_Final'],
        color = 'Continent',opacity=alt.condition(
        alt.datum.age < select_age.age,
        alt.value(0.7), alt.value(0.1))
).add_selection(select_age)
    return chart

def soc_viz_stats_hist(x_col, df):
    """Create a histogram of one chosen numerical feature.
    
    Parameters
    ----------
    x_col : str
        name of the feature in horizontal axis. 

        name of the feature in vertical axis.
    df : pandas dataframe
        The dataframe of the feature

    Returns
    -------
    Chart: altair.vegalite.v4.api.Chart
        an altair chart
    Examples
    --------
    >>> data = pd.read_excel("soccer_data.xlsx")
    >>> soc_viz_stats_hist('age', data)
    """
    assert isinstance(x_col,str) == True, "Column named used should be passed as string"
    assert (x_col in df.columns) == True, f'Column {x_col} does not exist in data frame'
    assert np.issubdtype(df[x_col].dtype, np.number) ==True, "Column used should be numeric"
    
    chart = alt.Chart(df).mark_bar().encode(
        x=alt.X(x_col, bin=alt.Bin(maxbins=30)),
        y='count()')
    return chart






# Author: Flora Ouedraogo
import pandas as pd 

def rankingplayers(df, *col_name):

    """Returns a sorted table of players based on which variables the user decide to use to rank players.
    This function can help users find the most and less important variables which can be used to compare 
    one player and the other in terms of performances. 
    
    Parameters
    ----------
    df  : pandas dataframe
        The dataframe of the soccer dataset    
    col_name : str
        The name of the columns which the user decide to use to rank the players one 
        
    Returns
    -------
    ranked_df: pandas dataframe
        This is the sorted data frame by column name
  
    Examples
    --------
    >>> rankingplayers(soccer_df, "Goals_total", "Assists_Total")
    """

    df = df.sort_values(by=[*col_name], ascending = False)
    df = pd.DataFrame(df["Name"])
    
    return df.reset_index(drop = True)


# Author: Vincent Ho
import pandas as pd
import altair as alt
from IPython.display import display


def find_team_stat(df, team_name, feature="age"):
    """Returns the descriptive statistic table and box plot of a particular team.

    This function can help users to find the statistical data of a particular team.
    A box plot is provided for visualization of the statistical data.
    Users can visualize a certain feature of the team by inputting it in the optional argument "feature".
    If no input is found, the default feature of the box plot will be the "age" of the user's inputted team.
    
    Parameters
    ----------
    df : pandas dataframe
        The dataframe of the soccer dataset. 
    team_name : str
        String in the dataframe. Users can input any string from the "Team" column.
    feature : str
        Optional argument. The default feature is "age". Users can input other numerical features.
    

    Returns
    -------
    dataframe
        Descriptive statistics include those that summarize the central tendency, 
        dispersion and shape of a dataset's distribution, excluding NaN values.
    
    Boxplot
        A visualization plot to show a certain descriptive statistic feature of a particular team.

    

    Examples
    --------
    >>> find_team_stat(soccer_df, "Manchester United", "Market_Value_Euros")
    """
    
    if not isinstance(df, pd.DataFrame):
        raise TypeError("The data must be in type of Pandas DataFrame.")
        
    if not isinstance(team_name, str):
            raise TypeError("Incorrect type in argument: `team_name`. Please pass in a string for `team_name`, and follow the team name format under column 'Team' in the dataset.")
            
    if not isinstance(feature, str):
            raise TypeError("Incorrect type in argument: `feature`. Please pass in a string for `feature`, and follow the name format of any numerical column in the dataset.") 
    
    team_df = df.query(f"Team==@team_name")
    
    display(team_df.describe())
    
    box_plot = alt.Chart(team_df).mark_boxplot().encode( x = feature, y = "Team")
    
    return box_plot


# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['socceranalysis']

package_data = \
{'': ['*']}

install_requires = \
['altair>=4.2.0',
 'display>=1.0.0,<2.0.0',
 'ipython>=8.8.0,<9.0.0',
 'matplotlib>=3.6.3',
 'numpy>=1.24.1',
 'openpyxl>=3.0.10',
 'pandas>=1.5.2',
 'panel>=0.14.2,<0.15.0',
 'python-semantic-release>=7.33.0,<8.0.0']

setup_kwargs = {
    'name': 'socceranalysis',
    'version': '0.1.3',
    'description': 'Doing soccer stats analysis.',
    'long_description': '[![docs](https://img.shields.io/docsrs/passing)](https://soccer-analysis-python.readthedocs.io/en/latest/)\n\n# socceranalysis\n\nsocceranalysis is a powerful Python package designed to make it easy to analyze and understand soccer statistics. With its set of functions, you can quickly obtain summary statistics for a particular team, identify outliers based on market value, rank players by goals per game and display different plots. The package is built in a way that allows user to easily customize the functions to their own interests, giving them the flexibility to analyze the data in a way that is most meaningful to them. Whether you\'re a coach, a sports journalist or an analyst, socceranalysis will help you unlock the insights hidden in your soccer data and make more informed decisions.\n\n## Functions\n\n\n1. `find_team_stat`: provides a quick and easy way to understand the descriptive statistics of a team. (https://github.com/UBC-MDS/socceranalysis_python/blob/main/src/socceranalysis/find_team_stat.py) \n\n2. `rankingplayers`:  Ranks players based on specific attributes (https://github.com/UBC-MDS/socceranalysis_python/blob/main/src/socceranalysis/playerranking.py)\n\n3. `get_outliers`: Identifes outliers using statistical methods (interquartile range or standard deviations) (https://github.com/UBC-MDS/socceranalysis_python/blob/main/src/socceranalysis/outlier_identification.py)\n\n4. `soc_viz_stats` :  Generates meaningful visualizations to help users understand and interpret the data (https://github.com/UBC-MDS/socceranalysis_python/blob/main/src/socceranalysis/viz_stats.py)\n* `soc_viz_stats_scatter` : Generate a scatter plot for two given numeric columns with a slider to control age \n* `soc_viz_stats_hist` :  Generate a histogram for one given numeric columns\n\n\n\n## Python ecosystem\nsocceranalysis can be used in conjunction with other popular Python packages such as [pandas](https://github.com/pandas-dev/pandas) and [scikit-learn](https://github.com/scikit-learn/scikit-learn) to perform more advanced data analysis and machine learning tasks. For example, users can use pandas to manipulate and clean their soccer data, and then use this package to perform specific soccer-related analysis on the cleaned data. Additionally, socceranalysis can be used in conjunction with scikit-learn for machine learning tasks on soccer data. They are designed to be a higher-level, more user-friendly and declarative interface based on [Altair](https://github.com/altair-viz/altair) for performing specific soccer-related analysis and visualization tasks. Users can perform similar visualization using [matplotlib](https://github.com/matplotlib/matplotlib). Overall, socceranalysis is a valuable addition to the Python ecosystem as it provides a specialized tool for analyzing and understanding soccer data without the need for writing complex code, this can be especially useful for users who may not have extensive experience with data analysis or visualization.\n\n\n\n## Installation\n\n```bash\n$ pip install socceranalysis\n\n# directly from test pypi\n$ pip install -i https://test.pypi.org/simple/ socceranalysis\n```\n\n## Usage\n\n\n###  find_team_stat\n```bash\nfrom socceranalysis.find_team_stat import *\ndata = pd.read_excel(\'soccer_data.xlsx\')\nfind_team_stat(data , "Manchester United", "Market_Value_Euros")\n```\n### get_outliers\n```bash \ndata = pd.read_excel(\'soccer_data.xlsx\')\nfrom  socceranalysis.outlier_identification import get_outliers\nget_outliers(data,"Wages_Euros","SD",3)\n```\n###  soc_viz_stats\n\n```bash\nfrom socceranalysis.viz_stats import *\n\n# scatter plots of two given columns\nsoc_viz_stats_scatter(\'age\',\'Goals_total\', data)\n# histogram of one given column\nsoc_viz_stats_hist(\'age\', data)\n```\n### playerranking\n```bash\nfrom socceranalysis.rankingplayers import *\ndata = pd.read_excel(\'soccer_data.xlsx\')\nrankingplayers(data, "Goals_total", "Assists_Total")\n```\n\n## Contributors\n\n|  \t Core contributor| Github.com username| \n|---------|---|\n|  Flora Ouedraogo |  @florawendy19 | \n|  Gaoxiang Wang |  @louiewang820 | \n|  Manvir Kohli | @manvirsingh96 |\n| Vincent Ho | @vincentho32 |\n\n## Contributing\n\nAuthors: Vincent Ho, Manvir Singh Kohli, Gaoxiang Wang, Flora Ouedraogo.\n\nInterested in contributing? Check out the contributing guidelines. Please note that this project is released with a Code of Conduct. By contributing to this project, you agree to abide by its terms.\n\n## License\n\n`socceranalysis` was created by Gaoxiang Wang, Manvir Kohli, Vincent Ho and Flora Ouedraogo. It is licensed under the terms of the MIT license.\n\n## Credits\n\n`socceranalysis` was created with [`cookiecutter`](https://cookiecutter.readthedocs.io/en/latest/) and the `py-pkgs-cookiecutter` [template](https://github.com/py-pkgs/py-pkgs-cookiecutter).\n',
    'author': 'Group 15 ',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/UBC-MDS/socceranalysis_python',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9',
}


setup(**setup_kwargs)




import numpy as np
import pandas as pd

def get_outliers(df,col, method = "SD",thresh=3):
    
    """Returns outliers in the dataset based on values of a variable

    This function identifies outliers in the dataset based on either of the following methods:
    
    1. Interquartile Range (IQR) Method: Identifies all values less than Q1 - 1.5*IQR and 
    greater than Q3 + 1.5*IQR where IQR = Q3-Q1, are identified as outliers.
    
    2. Mean and Standard Deviation Method: Identifies all values less than 
    mean - k*standard_deviation and greater than mean + k*standard_deviation as outliers.
    
    Parameters
    ----------
    df : dataframe
        Dataframe in  which outliers are to be identified.
    col : str
        Variable in the dataframe based on which outliers are to be identified.
    method : str
        Name of the outlier identification method to be used. 
        "IQR" for IQR method and "SD" for mean and standard deviation method.
    thresh : int
        The value of k in the Mean and Standard Deviation Method formula above.

    Returns
    -------
    dataframe
        Subset of original dataframe  containing only rows corresponding to outliers.

    Examples
    --------
    >>> get_outliers(df,"Wages_Euros","SD",3)
    """
    assert isinstance (df,pd.DataFrame) == True, "Input data must be a dataframe"
    assert isinstance(col,str) == True, "Column named used for outlier detection should be passed as string"
    assert (col in df.columns) == True, f"Column {col} does not exist in data frame"
    assert np.issubdtype(df[col].dtype, np.number) ==True, "Column used for outlier detection should be numeric"
    
    if method == "SD":
        mean = np.mean(df[col])
        std = np.std(df[col])
        outliers = df[(df[col] > mean + thresh*std) | (df[col] < mean - thresh*std)]
    
    elif method == "IQR":
        q1,q3 =  np.percentile(df[col], [25 ,75])
        iqr = q3 - q1
        outliers = df[(df[col] > q3 + 1.5*iqr) | (df[col] < q1 - 1.5*iqr)]
        
    return outliers
        
        
        
    
        